@@ -2,7 +2,7 @@
 """Test suite."""
 import pytest
 
-from axonius_api_client.api.parsers.constants import Operators, OperatorTypeMaps
+from axonius_api_client.constants.fields import Operators, OperatorTypeMaps
 from axonius_api_client.exceptions import NotFoundError
 
 
